@@ -3,9 +3,18 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
+import os
+
 from importlib import metadata
 
 
+# Set canonical URL from the Read the Docs Domain
+html_baseurl = os.environ.get("READTHEDOCS_CANONICAL_URL", "")
+
+# Tell Jinja2 templates the build is running on Read the Docs
+if os.environ.get("READTHEDOCS", "") == "True":
+    html_context = {"READTHEDOCS": True}
+
 # We want an image in the README and include the README in the docs.
 suppress_warnings = ["image.nonlocal_uri"]
 
@@ -22,6 +31,8 @@ extensions = [
     "sphinx.ext.intersphinx",
     "sphinx.ext.viewcode",
     "sphinxcontrib.mermaid",
+    "sphinxext.opengraph",
+    "sphinx_copybutton",
 ]
 
 myst_enable_extensions = [
@@ -31,6 +42,9 @@ myst_enable_extensions = [
 ]
 mermaid_init_js = "mermaid.initialize({startOnLoad:true,theme:'neutral'});"
 
+ogp_image = "_static/structlog_logo.png"
+
+
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ["_templates"]
 
@@ -43,7 +57,7 @@ master_doc = "index"
 # General information about the project.
 project = "structlog"
 author = "Hynek Schlawack"
-copyright = f"2013, { author }"
+copyright = f"2013, {author}"
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the
@@ -70,9 +84,11 @@ nitpick_ignore = [
     ("py:class", "PlainFileObserver"),
     ("py:class", "Processor"),
     ("py:class", "Styles"),
+    ("py:class", "structlog.dev._Styles"),
     ("py:class", "WrappedLogger"),
     ("py:class", "structlog.threadlocal.TLLogger"),
     ("py:class", "structlog.typing.EventDict"),
+    ("py:class", "ModuleType"),
 ]
 
 # If true, '()' will be appended to :func: etc. cross-reference text.
@@ -84,36 +100,14 @@ autodoc_typehints_description_target = "documented"
 
 # -- Options for HTML output --------------------------------------------------
 
-# The theme to use for HTML and HTML Help pages.  See the documentation for
-# a list of builtin themes.
-html_theme = "furo"
+html_theme = "sphinx_book_theme"
 html_theme_options = {}
-html_logo = "_static/structlog_logo_small_transparent.png"
+html_logo = "_static/structlog_logo.svg"
 html_static_path = ["_static"]
+html_css_files = ["custom.css"]
 
 htmlhelp_basename = "structlogdoc"
 
-_logo = (
-    "https://www.structlog.org/en/latest/_static/"
-    "structlog_logo_small_transparent.png"
-)
-_descr = (
-    "structlog makes logging in Python faster, less painful, and more "
-    "powerful by adding structure to your log entries."
-)
-_title = "structlog: Structured Logging for Python"
-rst_epilog = f"""\
-.. meta::
-    :property=og:type: website
-    :property=og:site_name: { _title }
-    :property=og:description: { _descr }
-    :property=og:author: Hynek Schlawack
-    :property=og:image: { _logo }
-    :twitter:title: { _title }
-    :twitter:image: { _logo }
-    :twitter:creator: @hynek
-"""
-
 latex_documents = [
     ("index", "structlog.tex", "structlog Documentation", "Author", "manual")
 ]
