@@ -7,9 +7,9 @@ import abc
 import pickle
 import warnings
 
-import pytest
+from unittest.mock import call
 
-from pretend import call, call_recorder, stub
+import pytest
 
 import structlog
 
@@ -26,6 +26,9 @@ from structlog._config import (
     get_logger,
     wrap_logger,
 )
+from structlog.typing import BindableLogger
+
+from .helpers import call_recorder, stub
 
 
 @pytest.fixture(name="proxy")
@@ -50,12 +53,32 @@ def test_lazy_logger_is_not_detected_as_abstract_method():
     See https://github.com/hynek/structlog/issues/229
     """
 
-    class Foo(metaclass=abc.ABCMeta):
+    class Foo(metaclass=abc.ABCMeta):  # noqa: B024
         log = structlog.get_logger()
 
     Foo()
 
 
+def test_lazy_logger_is_an_instance_of_bindable_logger():
+    """
+    The BoundLoggerLazyProxy returned by get_logger fulfills the BindableLogger
+    protocol.
+
+    See https://github.com/hynek/structlog/issues/560
+    """
+    assert isinstance(get_logger(), BindableLogger)
+
+
+def test_lazy_logger_context_is_initial_values():
+    """
+    If a user asks for _context (e.g., using get_context) return
+    initial_values.
+    """
+    logger = get_logger(context="a")
+
+    assert {"context": "a"} == structlog.get_context(logger)
+
+
 def test_default_context_class():
     """
     Default context class is dict.
@@ -416,7 +439,7 @@ class TestFunctions:
             configure_once()
 
         assert 1 == len(warns)
-        assert RuntimeWarning == warns[0].category
+        assert RuntimeWarning is warns[0].category
         assert "Repeated configuration attempted." == warns[0].message.args[0]
 
     def test_get_logger_configures_according_to_config(self):
@@ -442,4 +465,4 @@ class TestFunctions:
 
         get_logger("test").bind(x=42)
 
-        assert [call("test")] == factory.calls
+        assert [call("test")] == factory.call_args_list
