@@ -6,6 +6,7 @@
 import pickle
 
 import pytest
+import time_machine
 
 from structlog._config import _CONFIG
 from structlog._generic import BoundLogger
@@ -37,26 +38,11 @@ class TestGenericBoundLogger:
 
         assert "msg" in b.__dict__
 
-    def test_proxies_anything(self):
-        """
-        Anything that isn't part of BoundLoggerBase gets proxied to the correct
-        wrapped logger methods.
-        """
-        b = BoundLogger(
-            ReturnLogger(),
-            _CONFIG.default_processors,
-            _CONFIG.default_context_class(),
-        )
-
-        assert "log", "foo" == b.log("foo")
-        assert "gol", "bar" == b.gol("bar")
-
-    @pytest.mark.parametrize("proto", range(pickle.HIGHEST_PROTOCOL + 1))
+    @pytest.mark.parametrize("proto", range(3, pickle.HIGHEST_PROTOCOL + 1))
+    @time_machine.travel("2023-05-22 17:00", tick=False)
     def test_pickle(self, proto):
         """
         Can be pickled and unpickled.
-
-        Works only on Python 3: TypeError: can't pickle instancemethod objects
         """
         b = BoundLogger(
             ReturnLogger(),
