@@ -12,21 +12,21 @@ import os
 import sys
 
 from io import StringIO
-from typing import Any, Callable, Collection
+from typing import Any, Callable, Collection, Dict
+from unittest.mock import patch
 
 import pytest
 import pytest_asyncio
 
-from pretend import call_recorder, stub
-
 from structlog import (
     PrintLogger,
     ReturnLogger,
     configure,
     get_context,
+    wrap_logger,
 )
 from structlog._config import _CONFIG
-from structlog._log_levels import _NAME_TO_LEVEL, CRITICAL, WARN
+from structlog._log_levels import CRITICAL, NAME_TO_LEVEL, WARN
 from structlog.dev import ConsoleRenderer
 from structlog.exceptions import DropEvent
 from structlog.processors import JSONRenderer, KeyValueRenderer
@@ -44,12 +44,14 @@ from structlog.stdlib import (
     filter_by_level,
     get_logger,
     recreate_defaults,
+    render_to_log_args_and_kwargs,
     render_to_log_kwargs,
 )
 from structlog.testing import CapturedCall
 from structlog.typing import BindableLogger, EventDict
 
 from .additional_frame import additional_frame
+from .helpers import call_recorder, stub
 
 
 def build_bl(logger=None, processors=None, context=None):
@@ -114,7 +116,7 @@ class TestLoggerFactory:
         """
         logger = _FixedFindCallerLogger("test")
 
-        file_name, line_number, func_name = logger.findCaller()[:3]
+        file_name, _line_number, func_name = logger.findCaller()[:3]
 
         assert file_name == os.path.realpath(__file__)
         assert func_name == "test_deduces_correct_caller"
@@ -124,7 +126,7 @@ class TestLoggerFactory:
         If we ask for stack_info, it will returned.
         """
         logger = _FixedFindCallerLogger("test")
-        testing, is_, fun, stack_info = logger.findCaller(stack_info=True)
+        testing, is_, fun, stack_info = logger.findCaller(stack_info=True)  # noqa: RUF059
 
         assert "testing, is_, fun" in stack_info
 
@@ -133,7 +135,7 @@ class TestLoggerFactory:
         If we don't ask for stack_info, it won't be returned.
         """
         logger = _FixedFindCallerLogger("test")
-        testing, is_, fun, stack_info = logger.findCaller()
+        testing, is_, fun, stack_info = logger.findCaller()  # noqa: RUF059
 
         assert None is stack_info
 
@@ -193,7 +195,8 @@ class TestFilterByLevel:
 
 class TestBoundLogger:
     @pytest.mark.parametrize(
-        ("method_name"), ["debug", "info", "warning", "error", "critical"]
+        ("method_name"),
+        ["debug", "info", "warning", "error", "exception", "critical"],
     )
     def test_proxies_to_correct_method(self, method_name):
         """
@@ -203,13 +206,14 @@ class TestBoundLogger:
 
         assert method_name == getattr(bl, method_name)("event")
 
-    def test_proxies_exception(self):
+    def test_proxies_to_correct_method_special_cases(self):
         """
-        BoundLogger.exception is proxied to Logger.error.
+        Fatal maps to critical and warn to warning.
         """
         bl = BoundLogger(ReturnLogger(), [return_method_name], {})
 
-        assert "error" == bl.exception("event")
+        assert "warning" == bl.warn("event")
+        assert "critical" == bl.fatal("event")
 
     def test_proxies_log(self):
         """
@@ -225,7 +229,7 @@ class TestBoundLogger:
         Positional arguments supplied are proxied as kwarg.
         """
         bl = BoundLogger(ReturnLogger(), [], {})
-        args, kwargs = bl.debug("event", "foo", bar="baz")
+        _args, kwargs = bl.debug("event", "foo", bar="baz")
 
         assert "baz" == kwargs.get("bar")
         assert ("foo",) == kwargs.get("positional_args")
@@ -364,6 +368,20 @@ class TestBoundLogger:
             CapturedCall(method_name=meth, args=(), kwargs={"event": "Async!"})
         ] == cl.calls
 
+    async def test_async_log_methods_special_cases(self, cl):
+        """
+        afatal maps to critical.
+        """
+        bl = build_bl(cl, processors=[])
+
+        await bl.afatal("Async!")
+
+        assert [
+            CapturedCall(
+                method_name="critical", args=(), kwargs={"event": "Async!"}
+            )
+        ] == cl.calls
+
     async def test_alog(self, cl):
         """
         Alog logs async at the correct level.
@@ -476,7 +494,7 @@ class TestPositionalArgumentsFormatter:
 
 
 class TestAddLogLevelNumber:
-    @pytest.mark.parametrize(("level", "number"), _NAME_TO_LEVEL.items())
+    @pytest.mark.parametrize(("level", "number"), NAME_TO_LEVEL.items())
     def test_log_level_number_added(self, level, number):
         """
         The log level number is added to the event dict.
@@ -495,13 +513,16 @@ class TestAddLogLevel:
 
         assert "error" == event_dict["level"]
 
-    def test_log_level_alias_normalized(self):
+    @pytest.mark.parametrize(
+        ("alias", "normalized"), [("warn", "warning"), ("exception", "error")]
+    )
+    def test_log_level_alias_normalized(self, alias, normalized):
         """
         The normalized name of the log level is added to the event dict.
         """
-        event_dict = add_log_level(None, "warn", {})
+        event_dict = add_log_level(None, alias, {})
 
-        assert "warning" == event_dict["level"]
+        assert normalized == event_dict["level"]
 
 
 @pytest.fixture(name="make_log_record")
@@ -641,7 +662,7 @@ class TestExtraAdder:
         handler.setLevel(0)
         logger.addHandler(handler)
         logger.setLevel(0)
-        logging.warning("allow = %s", allow)
+        logging.warning("allow = %s", allow)  # noqa: LOG015
 
         event_dict = {"event": "Some text"}
         expected = self._copy_allowed(event_dict, extra_dict, allow)
@@ -675,8 +696,182 @@ class TestExtraAdder:
         }
 
 
-class TestRenderToLogKW:
-    def test_default(self):
+@pytest.fixture(name="stdlib_logger")
+def _stdlib_logger():
+    logger = logging.getLogger("test_logger")
+    logger.setLevel(logging.DEBUG)
+
+    yield logger
+
+    logging.basicConfig()
+
+
+class TestRenderToLogArgsAndKwargs:
+    def test_default(self, stdlib_logger: logging.Logger):
+        """
+        Passes `event` key from `event_dict` in the first positional argument
+        and handles otherwise empty `event_dict`.
+        """
+        method_name = "debug"
+        event = "message"
+        args, kwargs = render_to_log_args_and_kwargs(
+            stdlib_logger, method_name, {"event": event}
+        )
+
+        assert (event,) == args
+        assert {} == kwargs
+
+        with patch.object(stdlib_logger, "_log") as mock_log:
+            getattr(stdlib_logger, method_name)(*args, **kwargs)
+
+        mock_log.assert_called_once_with(logging.DEBUG, event, ())
+
+    def test_pass_remaining_event_dict_as_extra(
+        self, stdlib_logger: logging.Logger, event_dict: dict[str, Any]
+    ):
+        """
+        Passes remaining `event_dict` as `extra`.
+        """
+        expected_extra = event_dict.copy()
+
+        method_name = "info"
+        event = "message"
+        event_dict["event"] = event
+
+        args, kwargs = render_to_log_args_and_kwargs(
+            stdlib_logger, method_name, event_dict
+        )
+
+        assert (event,) == args
+        assert {"extra": expected_extra} == kwargs
+
+        with patch.object(stdlib_logger, "_log") as mock_log:
+            getattr(stdlib_logger, method_name)(*args, **kwargs)
+
+        mock_log.assert_called_once_with(
+            logging.INFO, event, (), extra=expected_extra
+        )
+
+    def test_pass_positional_args_from_event_dict_as_args(
+        self, stdlib_logger: logging.Logger, event_dict: dict[str, Any]
+    ):
+        """
+        Passes items from "positional_args" key from `event_dict` as positional
+        arguments.
+        """
+        expected_extra = event_dict.copy()
+
+        method_name = "warning"
+        event = "message: a = %s, b = %d"
+        positional_args = ("foo", 123)
+        event_dict["event"] = event
+        event_dict["positional_args"] = positional_args
+
+        args, kwargs = render_to_log_args_and_kwargs(
+            stdlib_logger, method_name, event_dict
+        )
+
+        assert (event, *(positional_args)) == args
+        assert {"extra": expected_extra} == kwargs
+
+        with patch.object(stdlib_logger, "_log") as mock_log:
+            getattr(stdlib_logger, method_name)(*args, **kwargs)
+
+        mock_log.assert_called_once_with(
+            logging.WARNING, event, positional_args, extra=expected_extra
+        )
+
+    def test_pass_kwargs_from_event_dict_as_kwargs(
+        self, stdlib_logger: logging.Logger, event_dict: dict[str, Any]
+    ):
+        """
+        Passes "exc_info", "stack_info", and "stacklevel" keys from `event_dict`
+        as keyword arguments.
+        """
+        expected_extra = event_dict.copy()
+
+        method_name = "info"
+        event = "message"
+        exc_info = True
+        stack_info = False
+        stacklevel = 2
+        event_dict["event"] = event
+        event_dict["exc_info"] = exc_info
+        event_dict["stack_info"] = stack_info
+        event_dict["stacklevel"] = stacklevel
+
+        args, kwargs = render_to_log_args_and_kwargs(
+            stdlib_logger, method_name, event_dict
+        )
+
+        assert (event,) == args
+        assert {
+            "exc_info": exc_info,
+            "stack_info": stack_info,
+            "stacklevel": stacklevel,
+            "extra": expected_extra,
+        } == kwargs
+
+        with patch.object(stdlib_logger, "_log") as mock_log:
+            getattr(stdlib_logger, method_name)(*args, **kwargs)
+
+        mock_log.assert_called_once_with(
+            logging.INFO,
+            event,
+            (),
+            exc_info=exc_info,
+            stack_info=stack_info,
+            stacklevel=stacklevel,
+            extra=expected_extra,
+        )
+
+    def test_integration(
+        self, stdlib_logger: logging.Logger, event_dict: dict[str, Any]
+    ):
+        """
+        `render_to_log_args_and_kwargs` with a wrapped logger calls the stdlib
+        logger correctly.
+
+        Reserved stdlib keyword arguments are in `logging.Logger._log`.
+        https://github.com/python/cpython/blob/60403a5409ff2c3f3b07dd2ca91a7a3e096839c7/Lib/logging/__init__.py#L1640
+        """
+        event = "message: a = %s, b = %d"
+        arg_1 = "foo"
+        arg_2 = 123
+        exc_info = False
+        stack_info = True
+        stacklevel = 3
+
+        struct_logger = wrap_logger(
+            stdlib_logger,
+            processors=[render_to_log_args_and_kwargs],
+            wrapper_class=BoundLogger,
+        )
+
+        with patch.object(stdlib_logger, "_log") as mock_log:
+            struct_logger.info(
+                event,
+                arg_1,
+                arg_2,
+                exc_info=exc_info,
+                stack_info=stack_info,
+                stacklevel=stacklevel,
+                **event_dict,
+            )
+
+        mock_log.assert_called_once_with(
+            logging.INFO,
+            event,
+            (arg_1, arg_2),
+            exc_info=exc_info,
+            stack_info=stack_info,
+            stacklevel=stacklevel,
+            extra=event_dict,
+        )
+
+
+class TestRenderToLogKwargs:
+    def test_default(self, stdlib_logger):
         """
         Translates `event` to `msg` and handles otherwise empty `event_dict`s.
         """
@@ -684,7 +879,13 @@ class TestRenderToLogKW:
 
         assert {"msg": "message", "extra": {}} == d
 
-    def test_add_extra_event_dict(self, event_dict):
+        # now check stdlib logger likes those kwargs
+        with patch.object(stdlib_logger, "_log") as mock_log:
+            stdlib_logger.info(**d)
+
+        mock_log.assert_called_once_with(logging.INFO, "message", (), extra={})
+
+    def test_add_extra_event_dict(self, event_dict, stdlib_logger):
         """
         Adds all remaining data from `event_dict` into `extra`.
         """
@@ -693,9 +894,17 @@ class TestRenderToLogKW:
 
         assert {"msg": "message", "extra": event_dict} == d
 
-    def test_handles_special_kw(self, event_dict):
+        # now check stdlib logger likes those kwargs
+        with patch.object(stdlib_logger, "_log") as mock_log:
+            stdlib_logger.info(**d)
+
+        mock_log.assert_called_once_with(
+            logging.INFO, "message", (), extra=event_dict
+        )
+
+    def test_handles_special_kw(self, event_dict, stdlib_logger):
         """
-        "exc_info", "stack_info", and "stackLevel" aren't passed as extras.
+        "exc_info", "stack_info", and "stacklevel" aren't passed as extras.
 
         Cf. https://github.com/hynek/structlog/issues/424
         """
@@ -704,22 +913,68 @@ class TestRenderToLogKW:
 
         event_dict["exc_info"] = True
         event_dict["stack_info"] = False
-        event_dict["stackLevel"] = 1
+        event_dict["stacklevel"] = 1
+        event_dict["stackLevel"] = 1  # not a reserved kw
 
         d = render_to_log_kwargs(None, None, event_dict)
-
-        assert {
+        expected = {
             "msg": "message",
             "exc_info": True,
             "stack_info": False,
-            "stackLevel": 1,
+            "stacklevel": 1,
             "extra": {
                 "b": [3, 4],
                 "x": 7,
                 "y": "test",
                 "z": (1, 2),
+                "stackLevel": 1,
             },
-        } == d
+        }
+
+        assert expected == d
+
+        # now check stdlib logger likes those kwargs
+        with patch.object(stdlib_logger, "_log") as mock_log:
+            stdlib_logger.info(**d)
+
+        expected.pop("msg")
+        mock_log.assert_called_once_with(
+            logging.INFO, "message", (), **expected
+        )
+
+    def test_integration_special_kw(self, event_dict, stdlib_logger):
+        """
+        render_to_log_kwargs with a wrapped logger calls the stdlib logger
+        correctly
+
+        reserved stdlib keywords are in logging.Logger._log
+        https://github.com/python/cpython/blob/ae7b17673f29efe17b416cbcfbf43b5b3ff5977c/Lib/logging/__init__.py#L1632
+        """
+        expected = {
+            "msg": "message",
+            "exc_info": True,
+            "stack_info": False,
+            "stacklevel": 1,
+            "extra": {**event_dict},
+        }
+
+        event_dict["exc_info"] = True
+        event_dict["stack_info"] = False
+        event_dict["stacklevel"] = 1
+
+        struct_logger = wrap_logger(
+            stdlib_logger,
+            processors=[render_to_log_kwargs],
+        )
+
+        # now check struct logger passes those kwargs to stdlib
+        with patch.object(stdlib_logger, "_log") as mock_log:
+            struct_logger.info("message", **event_dict)
+
+        expected.pop("msg")
+        mock_log.assert_called_once_with(
+            logging.INFO, "message", (), **expected
+        )
 
 
 @pytest.fixture(name="configure_for_processor_formatter")
@@ -844,7 +1099,7 @@ class TestProcessorFormatter:
         positional_args = {"foo": "bar"}
         logging.getLogger().info("okay %(foo)s", positional_args)
 
-        event_dict = test_processor.calls[0].args[2]
+        event_dict = test_processor.call_args_list[0].args[2]
 
         assert "positional_args" in event_dict
         assert positional_args == event_dict["positional_args"]
@@ -886,7 +1141,7 @@ class TestProcessorFormatter:
 
         assert (
             "",
-            "foo                            [sample-name]  [in test_foreign_pr"
+            "foo                            [sample-name] [in test_foreign_pr"
             "e_chain_add_logger_name]\n",
         ) == capsys.readouterr()
 
@@ -925,7 +1180,7 @@ class TestProcessorFormatter:
         except Exception:
             logging.getLogger().exception("okay")
 
-        event_dict = test_processor.calls[0].args[2]
+        event_dict = test_processor.call_args_list[0].args[2]
 
         assert "exc_info" in event_dict
         assert isinstance(event_dict["exc_info"], tuple)
@@ -953,7 +1208,7 @@ class TestProcessorFormatter:
         except Exception:
             logging.getLogger().error("okay")
 
-        event_dict = test_processor.calls[0].args[2]
+        event_dict = test_processor.call_args_list[0].args[2]
 
         assert MyError is event_dict["exc_info"][0]
 
@@ -976,9 +1231,9 @@ class TestProcessorFormatter:
 
         logger.info("meh")
 
-        assert 1 == len(handler2.handle.calls)
+        assert 1 == len(handler2.handle.call_args_list)
 
-        handler2_record = handler2.handle.calls[0].args[0]
+        handler2_record = handler2.handle.call_args_list[0].args[0]
 
         assert "meh" == handler2_record.msg
 
@@ -1123,6 +1378,107 @@ class TestProcessorFormatter:
             {"foo": "bar", "_record": "foo", "_from_structlog": True},
         )
 
+    def test_non_string_message_warning(self):
+        """
+        A warning is raised if the last processor in
+        ProcessorFormatter.processors doesn't return a string.
+        """
+        configure_logging(None)
+        logger = logging.getLogger()
+
+        formatter = ProcessorFormatter(
+            processors=[lambda *args, **kwargs: {"foo": "bar"}],
+        )
+        logger.handlers[0].setFormatter(formatter)
+
+        with pytest.warns(
+            RuntimeWarning,
+            match="The last processor in ProcessorFormatter.processors must return a string",
+        ):
+            logger.info("baz")
+
+    def test_logrecord_exc_info(self):
+        """
+        LogRecord.exc_info is set consistently for structlog and non-structlog
+        log records.
+        """
+        configure_logging(None)
+
+        # This doesn't test ProcessorFormatter itself directly, but it's
+        # relevant to setups where ProcessorFormatter is used, i.e. where
+        # handlers will receive LogRecord objects that come from both structlog
+        # and non-structlog loggers.
+
+        records: Dict[  # noqa: UP006 - dict isn't generic until Python 3.9
+            str, logging.LogRecord
+        ] = {}
+
+        class DummyHandler(logging.Handler):
+            def emit(self, record):
+                # Don't do anything; just store the record in the records dict
+                # by its message, so we can assert things about it.
+                if isinstance(record.msg, dict):
+                    records[record.msg["event"]] = record
+                else:
+                    records[record.msg] = record
+
+        stdlib_logger = logging.getLogger()
+        structlog_logger = get_logger()
+
+        # It doesn't matter which logger we add the handler to here.
+        stdlib_logger.addHandler(DummyHandler())
+
+        try:
+            raise Exception("foo")
+        except Exception:
+            stdlib_logger.exception("bar")
+            structlog_logger.exception("baz")
+
+        stdlib_record = records.pop("bar")
+
+        assert "bar" == stdlib_record.msg
+        assert stdlib_record.exc_info
+        assert Exception is stdlib_record.exc_info[0]
+        assert ("foo",) == stdlib_record.exc_info[1].args
+
+        structlog_record = records.pop("baz")
+
+        assert "baz" == structlog_record.msg["event"]
+        assert True is structlog_record.msg["exc_info"]
+        assert structlog_record.exc_info
+        assert Exception is structlog_record.exc_info[0]
+        assert ("foo",) == structlog_record.exc_info[1].args
+
+        assert not records
+
+    def test_use_get_message_false(self):
+        """
+        If use_get_message_is False, the event is obtained using
+        str(record.msg) instead of calling record.getMessage. That means
+        positional formatting is not performed.
+        """
+        event_dicts = []
+
+        def capture(_, __, ed):
+            event_dicts.append(ed.copy())
+
+            return str(ed)
+
+        proc = ProcessorFormatter(processors=[capture], use_get_message=False)
+
+        record = logging.LogRecord(
+            "foo",
+            logging.INFO,
+            "path.py",
+            42,
+            "le msg: %s",
+            ("keep separate",),
+            None,
+        )
+
+        assert proc.format(record)
+        assert "le msg: %s" == event_dicts[0]["event"]
+
 
 @pytest_asyncio.fixture(name="abl")
 async def _abl(cl):
@@ -1140,28 +1496,36 @@ class TestAsyncBoundLogger:
             CapturedCall(method_name="info", args=(), kwargs={"event": "test"})
         ] == cl.calls
 
-    @pytest.mark.asyncio()
+    @pytest.mark.asyncio
     async def test_protocol(self, abl):
         """
         AsyncBoundLogger is a proper BindableLogger.
         """
         assert isinstance(abl, BindableLogger)
 
-    @pytest.mark.asyncio()
+    @pytest.mark.asyncio
     async def test_correct_levels(self, abl, cl, stdlib_log_method):
         """
         The proxy methods call the correct upstream methods.
         """
         await getattr(abl.bind(foo="bar"), stdlib_log_method)("42")
 
-        aliases = {"exception": "error", "warn": "warning"}
+        aliases = {"warn": "warning"}
 
-        alias = aliases.get(stdlib_log_method)
-        expect = alias if alias else stdlib_log_method
+        expect = aliases.get(stdlib_log_method, stdlib_log_method)
 
         assert expect == cl.calls[0].method_name
 
-    @pytest.mark.asyncio()
+    @pytest.mark.asyncio
+    async def test_correct_level_fatal(self, abl, cl):
+        """
+        fatal, that I have no idea why we support, maps to critical.
+        """
+        await abl.bind(foo="bar").fatal("42")
+
+        assert "critical" == cl.calls[0].method_name
+
+    @pytest.mark.asyncio
     async def test_log_method(self, abl, cl):
         """
         The `log` method is proxied too.
@@ -1170,7 +1534,7 @@ class TestAsyncBoundLogger:
 
         assert "error" == cl.calls[0].method_name
 
-    @pytest.mark.asyncio()
+    @pytest.mark.asyncio
     async def test_exception(self, abl, cl):
         """
         `exception` makes sure 'exc_info" is set, if it's not set already.
@@ -1185,7 +1549,7 @@ class TestAsyncBoundLogger:
         assert ValueError is ei[0]
         assert ("omg",) == ei[1].args
 
-    @pytest.mark.asyncio()
+    @pytest.mark.asyncio
     async def test_exception_do_not_overwrite(self, abl, cl):
         """
         `exception` leaves exc_info be, if it's set.
@@ -1202,7 +1566,7 @@ class TestAsyncBoundLogger:
         ei = cl.calls[0].kwargs["exc_info"]
         assert (o1, o2, o3) == ei
 
-    @pytest.mark.asyncio()
+    @pytest.mark.asyncio
     async def test_bind_unbind(self, cl):
         """
         new/bind/unbind/try_unbind are correctly propagated.
@@ -1233,7 +1597,7 @@ class TestAsyncBoundLogger:
         assert {} == l5._context
         assert l4 is not l5
 
-    @pytest.mark.asyncio()
+    @pytest.mark.asyncio
     async def test_integration(self, capsys):
         """
         Configure and log an actual entry.
