@@ -7,27 +7,37 @@ import json
 
 import pytest
 
-from pretend import call_recorder
-from twisted.python.failure import Failure
-from twisted.python.log import ILogObserver
-
 from structlog import ReturnLogger
 from structlog._config import _CONFIG
 from structlog.processors import KeyValueRenderer
-from structlog.twisted import (
-    BoundLogger,
-    EventAdapter,
-    JSONLogObserverWrapper,
-    JSONRenderer,
-    LoggerFactory,
-    PlainFileLogObserver,
-    ReprWrapper,
-    _extractStuffAndWhy,
-    plainJSONStdOutLogger,
-)
+
+from .helpers import call_recorder
+
+
+try:
+    from twisted.python.failure import Failure
+    from twisted.python.log import ILogObserver
+
+    from structlog.twisted import (
+        BoundLogger,
+        EventAdapter,
+        JSONLogObserverWrapper,
+        JSONRenderer,
+        LoggerFactory,
+        PlainFileLogObserver,
+        ReprWrapper,
+        _extractStuffAndWhy,
+        plainJSONStdOutLogger,
+    )
+
+except ImportError:
+    pytest.skip(allow_module_level=True)
 
 
 def test_LoggerFactory():
+    """
+    Logger factory ultimately returns twisted.python.log for output.
+    """
     from twisted.python import log
 
     assert log is LoggerFactory()()
@@ -77,18 +87,15 @@ class TestBoundLogger:
         except ValueError:
             # Use str() for comparison to avoid tricky
             # deep-compares of Failures.
-            assert (
-                str(
-                    (
-                        (),
-                        {
-                            "_stuff": Failure(ValueError()),
-                            "_why": "foo=42 event='event'",
-                        },
-                    )
+            assert str(
+                (
+                    (),
+                    {
+                        "_stuff": Failure(ValueError()),
+                        "_why": "foo=42 event='event'",
+                    },
                 )
-                == str(bl.err("event", foo=42))
-            )
+            ) == str(bl.err("event", foo=42))
 
 
 class TestExtractStuffAndWhy:
@@ -96,7 +103,10 @@ class TestExtractStuffAndWhy:
         """
         Raise ValueError if both _stuff and event contain exceptions.
         """
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(
+            ValueError,
+            match="Both _stuff and event contain an Exception/Failure",
+        ):
             _extractStuffAndWhy(
                 {
                     "_stuff": Failure(ValueError()),
@@ -104,20 +114,15 @@ class TestExtractStuffAndWhy:
                 }
             )
 
-        assert (
-            "Both _stuff and event contain an Exception/Failure."
-            == e.value.args[0]
-        )
-
     def test_failsOnConflictingEventAnd_why(self):
         """
         Raise ValueError if both _why and event are in the event_dict.
         """
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(
+            ValueError, match="Both `_why` and `event` supplied"
+        ):
             _extractStuffAndWhy({"_why": "foo", "event": "bar"})
 
-        assert "Both `_why` and `event` supplied." == e.value.args[0]
-
     def test_handlesFailures(self):
         """
         Extracts failures and events.
@@ -144,6 +149,9 @@ class TestEventAdapter:
     """
 
     def test_EventAdapterFormatsLog(self):
+        """
+        EventAdapter formats log entries correctly.
+        """
         la = EventAdapter(_render_repr)
 
         assert "{'foo': 'bar'}" == la(None, "msg", {"foo": "bar"})
@@ -208,13 +216,16 @@ class TestEventAdapter:
         )
 
     def test_catchesConflictingEventAnd_why(self):
+        """
+        Passing both _why and event raises a ValueError.
+        """
         la = EventAdapter(_render_repr)
 
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(
+            ValueError, match="Both `_why` and `event` supplied"
+        ):
             la(None, "err", {"event": "someEvent", "_why": "someReason"})
 
-        assert "Both `_why` and `event` supplied." == e.value.args[0]
-
 
 @pytest.fixture
 def jr():
@@ -260,6 +271,9 @@ class TestJSONRenderer:
         )
 
     def test_handlesFailure(self, jr):
+        """
+        JSONRenderer renders failures correctly.
+        """
         rv = jr(None, "err", {"event": Failure(ValueError())})[0][0].string
 
         assert "Failure: builtins.ValueError" in rv
@@ -283,9 +297,15 @@ class TestReprWrapper:
 
 class TestPlainFileLogObserver:
     def test_isLogObserver(self, sio):
+        """
+        PlainFileLogObserver is an ILogObserver.
+        """
         assert ILogObserver.providedBy(PlainFileLogObserver(sio))
 
     def test_writesOnlyMessageWithLF(self, sio):
+        """
+        PlainFileLogObserver writes only the message and a line feed.
+        """
         PlainFileLogObserver(sio)(
             {"system": "some system", "message": ("hello",)}
         )
@@ -295,6 +315,9 @@ class TestPlainFileLogObserver:
 
 class TestJSONObserverWrapper:
     def test_IsAnObserver(self):
+        """
+        JSONLogObserverWrapper is an ILogObserver.
+        """
         assert ILogObserver.implementedBy(JSONLogObserverWrapper)
 
     def test_callsWrappedObserver(self):
@@ -304,7 +327,7 @@ class TestJSONObserverWrapper:
         o = call_recorder(lambda *a, **kw: None)
         JSONLogObserverWrapper(o)({"message": ("hello",)})
 
-        assert 1 == len(o.calls)
+        assert 1 == len(o.call_args_list)
 
     def test_jsonifiesPlainLogEntries(self):
         """
@@ -313,7 +336,7 @@ class TestJSONObserverWrapper:
         """
         o = call_recorder(lambda *a, **kw: None)
         JSONLogObserverWrapper(o)({"message": ("hello",), "system": "-"})
-        msg = json.loads(o.calls[0].args[0]["message"][0])
+        msg = json.loads(o.call_args_list[0].args[0]["message"][0])
 
         assert msg == {"event": "hello", "system": "-"}
 
@@ -331,4 +354,7 @@ class TestJSONObserverWrapper:
 
 class TestPlainJSONStdOutLogger:
     def test_isLogObserver(self):
+        """
+        plainJSONStdOutLogger is an ILogObserver.
+        """
         assert ILogObserver.providedBy(plainJSONStdOutLogger())
